/**
 * string.h: Various string structures and functions.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005 Atheme Development Group
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: string.h,v 1.1 2007/09/22 14:27:26 pippijn Exp $
 */

#ifndef ERMYTH_STRING_H
#define ERMYTH_STRING_H

#include <cstddef>
#include <cstring>

enum match_rule
{
  MATCH_RFC1459,
  MATCH_ASCII
};

extern match_rule match_mapping;

extern const unsigned int charattrs[];

static const unsigned int C_ALPHA = 0x00000001;
static const unsigned int C_DIGIT = 0x00000002;

inline bool IsLower    (unsigned char c) { return c > 0x5f;                   }
inline bool IsUpper    (unsigned char c) { return !IsLower (c);               }
inline bool IsAlpha    (unsigned char c) { return (charattrs[c] & C_ALPHA);   }
inline bool IsDigit    (unsigned char c) { return (charattrs[c] & C_DIGIT);   }
inline bool IsAlphaNum (unsigned char c) { return IsAlpha (c) || IsDigit (c); }
inline bool IsNon      (unsigned char c) { return !IsAlphaNum (c);            }

extern const unsigned char ToLowerTab[];
extern const unsigned char ToUpperTab[];

extern void set_match_mapping (match_rule);

extern int ToLower (int);
extern int ToUpper (int);

#ifndef HAVE_STRLCAT
extern size_t strlcat (char *dest, char const *src, size_t count);
#endif
#ifndef HAVE_STRLCPY
extern size_t strlcpy (char *dest, char const *src, size_t count);
#endif

extern char const *strip (char *line);

extern int irccasecmp (char const * const s1, char const * const s2);
extern int ircncasecmp (char const * const s1, char const * const s2, int maxlen);

/* static string */
struct ststr
{
  ststr (char const * const initstr)
  : m_rep (initstr)
  {
  }

  ststr (ststr &initstr)
  : m_rep (initstr)
  {
  }

  char const *c_str () const
  {
    return m_rep;
  }

  operator char const * () const
  {
    return c_str ();
  }

private:
  char const * const m_rep;
};

// dynamic string
struct dynstr
{
  dynstr (size_t initlength = 32)
  : m_rep (new char[initlength]), pos (0), length (initlength)
  {
  }

  ~dynstr ()
  {
    delete[] m_rep;
  }

  void clear ()
  {
    m_rep[0] = '\0';
    pos = 0;
  }

  char const *c_str () const
  {
    return m_rep;
  }

  operator char const * () const
  {
    return c_str ();
  }

  size_t size ()
  {
    return length;
  }

  dynstr &operator = (char const *rhs)
  {
    clear ();
    add (rhs, strlen (rhs));
    return *this;
  }

  int add (char const c);
  int add (char const * const src, size_t size);

  template<int N>
  int add (char const (&src)[N])
  {
    return add (src, N);
  }

private:
  char *m_rep;
  size_t pos;
  size_t length;
};

#endif
